@@ -139,6 +139,30 @@ class SelectorTestCase(unittest.TestCase):
 
         self.assertEqual(sel.xpath('//div/text()').extract_first(default='missing'), 'missing')
 
+    def test_selector_get_alias(self):
+        """Test if get() returns extracted value on a Selector"""
+        body = u'<ul><li id="1">1</li><li id="2">2</li><li id="3">3</li></ul>'
+        sel = self.sscls(text=body)
+
+        self.assertEqual(sel.xpath('//ul/li[position()>1]')[0].get(), u'<li id="2">2</li>')
+        self.assertEqual(sel.xpath('//ul/li[position()>1]/text()')[0].get(), u'2')
+
+    def test_selector_getall_alias(self):
+        """Test if get() returns extracted value on a Selector"""
+        body = u'<ul><li id="1">1</li><li id="2">2</li><li id="3">3</li></ul>'
+        sel = self.sscls(text=body)
+
+        self.assertListEqual(sel.xpath('//ul/li[position()>1]')[0].getall(), [u'<li id="2">2</li>'])
+        self.assertListEqual(sel.xpath('//ul/li[position()>1]/text()')[0].getall(), [u'2'])
+
+    def test_selectorlist_get_alias(self):
+        """Test if get() returns first element for a selection call"""
+        body = u'<ul><li id="1">1</li><li id="2">2</li><li id="3">3</li></ul>'
+        sel = self.sscls(text=body)
+
+        self.assertEqual(sel.xpath('//ul/li').get(), u'<li id="1">1</li>')
+        self.assertEqual(sel.xpath('//ul/li/text()').get(), u'1')
+
     def test_re_first(self):
         """Test if re_first() returns first matched element"""
         body = u'<ul><li id="1">1</li><li id="2">2</li></ul>'
@@ -156,6 +180,18 @@ class SelectorTestCase(unittest.TestCase):
         self.assertEqual(sel.xpath('/ul/li/text()').re_first('\w+'), None)
         self.assertEqual(sel.xpath('/ul/li[@id="doesnt-exist"]/text()').re_first('\d'), None)
 
+        self.assertEqual(sel.re_first(r'id="(\d+)'), '1')
+        self.assertEqual(sel.re_first(r'foo'), None)
+        self.assertEqual(sel.re_first(r'foo', default='bar'), 'bar')
+
+    def test_extract_first_default(self):
+        """Test if re_first() returns default value when no results found"""
+        body = u'<ul><li id="1">1</li><li id="2">2</li></ul>'
+        sel = self.sscls(text=body)
+
+        self.assertEqual(sel.xpath('//div/text()').re_first('\w+', default='missing'), 'missing')
+        self.assertEqual(sel.xpath('/ul/li/text()').re_first('\w+', default='missing'), 'missing')
+
     def test_select_unicode_query(self):
         body = u"<p><input name='\xa9' value='1'/></p>"
         sel = self.sscls(text=body)
@@ -237,6 +273,31 @@ class SelectorTestCase(unittest.TestCase):
                          ["<li>four</li>", "<li>five</li>", "<li>six</li>"])
         self.assertEqual(divtwo.xpath("./li").extract(), [])
 
+    def test_selectorlist_getall_alias(self):
+        """Nested selector tests using getall()"""
+        body = u"""<body>
+                    <div class='one'>
+                      <ul>
+                        <li>one</li><li>two</li>
+                      </ul>
+                    </div>
+                    <div class='two'>
+                      <ul>
+                        <li>four</li><li>five</li><li>six</li>
+                      </ul>
+                    </div>
+                  </body>"""
+
+        x = self.sscls(text=body)
+        divtwo = x.xpath('//div[@class="two"]')
+        self.assertEqual(divtwo.xpath("//li").getall(),
+                         ["<li>one</li>", "<li>two</li>", "<li>four</li>", "<li>five</li>", "<li>six</li>"])
+        self.assertEqual(divtwo.xpath("./ul/li").getall(),
+                         ["<li>four</li>", "<li>five</li>", "<li>six</li>"])
+        self.assertEqual(divtwo.xpath(".//li").getall(),
+                         ["<li>four</li>", "<li>five</li>", "<li>six</li>"])
+        self.assertEqual(divtwo.xpath("./li").getall(), [])
+
     def test_mixed_nested_selectors(self):
         body = u'''<body>
                     <div id=1>not<span>me</span></div>
@@ -394,6 +455,41 @@ class SelectorTestCase(unittest.TestCase):
                          ["John", "Paul"])
         self.assertEqual(x.xpath("//ul/li").re("Age: (\d+)"),
                          ["10", "20"])
+        
+        # Test named group, hit and miss
+        x = self.sscls(text=u'foobar')
+        self.assertEqual(x.re('(?P<extract>foo)'), ['foo'])
+        self.assertEqual(x.re('(?P<extract>baz)'), [])
+
+        # A purposely constructed test for an edge case
+        x = self.sscls(text=u'baz')
+        self.assertEqual(x.re('(?P<extract>foo)|(?P<bar>baz)'), [])
+
+    def test_re_replace_entities(self):
+        body = u"""<script>{"foo":"bar &amp; &quot;baz&quot;"}</script>"""
+        x = self.sscls(text=body)
+        
+        name_re = re.compile('{"foo":(.*)}')
+
+        # by default, only &amp; and &lt; are preserved ;
+        # other entities are converted
+        expected = u'"bar &amp; "baz""'
+        self.assertEqual(x.xpath("//script/text()").re(name_re), [expected])
+        self.assertEqual(x.xpath("//script").re(name_re), [expected])
+        self.assertEqual(x.xpath("//script/text()")[0].re(name_re), [expected])
+        self.assertEqual(x.xpath("//script")[0].re(name_re), [expected])
+
+        # check that re_first() works the same way for single value output
+        self.assertEqual(x.xpath("//script").re_first(name_re), expected)
+        self.assertEqual(x.xpath("//script")[0].re_first(name_re), expected)
+
+        # switching off replace_entities will preserve &quot; also
+        expected = u'"bar &amp; &quot;baz&quot;"'
+        self.assertEqual(x.xpath("//script/text()").re(name_re, replace_entities=False), [expected])
+        self.assertEqual(x.xpath("//script")[0].re(name_re, replace_entities=False), [expected])
+
+        self.assertEqual(x.xpath("//script/text()").re_first(name_re, replace_entities=False), expected)
+        self.assertEqual(x.xpath("//script")[0].re_first(name_re, replace_entities=False), expected)
 
     def test_re_intl(self):
         body = u'<div>Evento: cumplea\xf1os</div>'
@@ -435,6 +531,10 @@ class SelectorTestCase(unittest.TestCase):
     def test_empty_bodies_shouldnt_raise_errors(self):
         self.sscls(text=u'').xpath('//text()').extract()
 
+    def test_bodies_with_comments_only(self):
+        sel = self.sscls(text=u'<!-- hello world -->', base_url='http://example.com')
+        self.assertEquals(u'http://example.com', sel.root.base)
+
     def test_null_bytes_shouldnt_raise_errors(self):
         text = u'<root>pre\x00post</root>'
         self.sscls(text).xpath('//text()').extract()
